#
# Copyright (c) 2004-2007 - Consultas, PKG.fr
#
# This file is part of A2P.
#
# A2P is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# A2P is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with A2P; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# $Id: OCD.pm 3 2007-10-18 16:20:19Z guillaume $
#
# Class to export Object Container Data members
#
# cf ref #2, p. 276
#

package AFPDS::MODCA::OCD ;

use strict ;
use A2P::Globals ;
use A2P::Syslog ;
use AFPDS::MODCA::Common ;

BEGIN {
    our $VERSION = sprintf "%s", q$Rev: 1007 $ =~ /([0-9.]+)\s+/ ;
}
our $VERSION ;
our @ISA = ("AFPDS::MODCA");
our $IDENTS ;

sub _ID { 0xD3EE92 }

sub new {
    my $class = shift ;
    &Debug("new " . __PACKAGE__ . " v$VERSION object");

    my $self = {
        FLUX     => 0,        # Associated flux
        IDENT    => _ID,      # MO:DCA identity
        FLAG     => 0,        # MO:DCA flags
        RESERVED => 0,        # MO:DCA reserved
        LONG     => 0,        # MO:DCA buffer length
        BUFFER   => ''        # MO:DCA content buffer
    };

    return bless $self , $class ;
}

sub validate {
    my $self = shift ;

    &Debug("Found OCD for integrated resource data");
    &UPSTAT('GET-RESOURCE-DATA');

    # Just control the autorized size of buffer
    return ( 251, "Not authorized " . $self->{LONG} . " OCD size" )
        if ( $self->{LONG} < 0 or $self->{LONG} > 32759 );

    # Check FLUX is a Flux object
    return ( 251, "OCD needs to be used with Flux object" )
        unless ( ref($self->{FLUX}) =~ /^AFPDS::Flux$/ );

    return () ;
}

sub create {
    my $self = shift ;

    if ($self->{LONG}) {
        my $flux = $self->{FLUX} ;

        my $object = $flux->get_resource ;
        return ( 252, "No object container available for datas" )
            unless ( $object );

        my $struct = $object->{STRUCT} || 0 ;
        return ( 253, "Unexpected '$struct' structure found for container" )
            unless ( $struct == 0xDC00 );

        # Initialize buffer if not exist
        $object->{RESOURCE_BUFFER} = ""
            unless (exists($object->{RESOURCE_BUFFER}));

        # Concatenate current buffers
        $object->{RESOURCE_BUFFER} .= $self->{BUFFER} ;

    } else {
        &Debug("Empty object container data found");
    }

    return () ;
}

&Debug("Module " . __PACKAGE__ . " v$VERSION loaded");

( $IDENTS->{&_ID} ) = __PACKAGE__ =~ /(\w+)$/ ;
